import { Socket, io } from "socket.io-client";
import { EVENTS } from "../constants";

export type User = {
  id: string;
  name: string;
};

export type ReplyMessagePayload = { fromId: string; message: string };
type LiveUsersCallback = (users: User[]) => void;
type ReplyCallback = (payload: ReplyMessagePayload) => void;

export class SocketClient {
  socket: Socket;

  constructor(url: string, autoConnect: boolean = false) {
    this.socket = io(url, { autoConnect });
  }

  connect(userName: string) {
    this.socket.connect();

    this.socket.on(EVENTS.CONNECT, () => {
      this.requsetToJoinRoom(userName);
    });
  }

  listenLiveUsers(onLiveUsersChange: LiveUsersCallback) {
    this.socket.on(EVENTS.LIVE_USERS, (liveUsers: User[]) => {
      const users = liveUsers.filter((lu: User) => lu.id !== this.socket.id);
      onLiveUsersChange(users);
    });
  }

  listenReply(onReply: ReplyCallback) {
    this.socket.on(EVENTS.REPLY, (payload: ReplyMessagePayload) => {
      onReply(payload);
    });
  }

  sendMessage(toId: string, message: string) {
    this.socket.emit(EVENTS.MESSAGE, {
      toId,
      message,
      fromId: this.socket.id,
    });
  }

  /**
   * Private methods
   */
  private requsetToJoinRoom(userName: string) {
    this.socket.emit(EVENTS.JOIN, { id: this.socket.id, name: userName });
  }
}
