import { SocketUsers } from "./SocketUsers";
import { EVENTS } from "../constants";
import { IChatData, ISocketUser } from "../types";

export class SocketServer {
  private _io: any;
  private _socket: any;
  private _liveUsers: SocketUsers;

  constructor(server: any) {
    this._io = server;
    this._liveUsers = new SocketUsers();
  }

  startListen() {
    this._io.on(EVENTS.CONNECTION, (socket: any) => {
      this._socket = socket;

      socket.on(EVENTS.JOIN, this.joinRoom.bind(this));
      socket.on(EVENTS.MESSAGE, this.onMessageReceived.bind(this));
      socket.on(EVENTS.DISCONNECT, () => {
        this._liveUsers.remove(socket.id);
        this.emitConnectedUsers();
      });
    });
  }

  joinRoom({ id, name }: ISocketUser) {
    this._socket.join(id);

    if (!this._liveUsers.isExist(id)) {
      this._liveUsers.add({ id, name });
      this.emitConnectedUsers();
    }
  }

  onMessageReceived({ toId, fromId, message }: IChatData) {
    this.sendMessage(toId, fromId, message);
  }

  emitConnectedUsers() {
    this._io.emit(EVENTS.LIVE_USERS, this._liveUsers.getAll());
  }

  sendMessage(toId: string, fromId: string, message: string) {
    this._io.to(toId).emit(EVENTS.REPLY, { fromId, message });
  }
}
